import collections
from threading import Thread

import cv2

# inspiration: https://stackoverflow.com/questions/57592857/python-opencv-mulithreading-with-rtsp-feeds


class VideoStreamReader:
    def __init__(self, src, deque_size=1):
        self.src = src
        # Start the thread to read frames from the video stream
        self.deque = collections.deque(maxlen=deque_size)
        self.load_network_stream()

        # Start background frame grabbing
        self.get_frame_thread = Thread(target=self.get_frame, args=())
        self.get_frame_thread.daemon = True
        self.get_frame_thread.start()

    def load_network_stream(self):
        """
        Verifies stream link and open new stream if valid.
        """

        def load_network_stream_thread():
            if self.verify_network_stream(self.src):
                self.capture = cv2.VideoCapture(self.src)
                self.online = True

        self.load_stream_thread = Thread(target=load_network_stream_thread, args=())
        self.load_stream_thread.daemon = True
        self.load_stream_thread.start()

    def verify_network_stream(self, link):
        """
        Attempts to receive a frame from given link.
        """
        cap = cv2.VideoCapture(link)
        if not cap.isOpened():
            return False
        cap.release()
        return True

    def get_frame(self):
        """
        Reads frame, resizes, and converts image to pixmap.
        """
        while True:
            try:
                if self.capture.isOpened() and self.online:
                    # Read next frame from stream and insert into deque
                    status, frame = self.capture.read()
                    if status:
                        self.deque.append(frame)
                    else:
                        self.capture.release()
                        self.online = False
                else:
                    # Attempt to reconnect
                    print("attempting to reconnect", self.camera_stream_link)
                    self.load_network_stream()

            except AttributeError:
                pass

    def get_video_frame(self):
        try:
            return True, self.deque[-1]
        except:
            return False, None
